Index: utils/translation.py
===================================================================
--- utils/translation.py	(revision 767)
+++ utils/translation.py	(working copy)
@@ -42,11 +42,14 @@
         self.set_output_charset('utf-8')
         self.__app = '?.?.?'
         self.__language = '??'
-    
+
     def set_app_and_language(self, app, language):
         self.__app = app
         self.__language = language
-    
+
+    def language(self):
+        return self.__language
+
     def __repr__(self):
         return "<DjangoTranslation app:%s lang:%s>" % (self.__app, self.__language)
 
@@ -66,7 +69,7 @@
     t = _translations.get((appname, language), None)
     if t is not None:
         return t
-    
+
     from django.conf import settings
 
     globalpath = os.path.join(os.path.dirname(settings.__file__), 'locale')
@@ -120,6 +123,15 @@
     """
     del _active[currentThread()]
 
+def get_language():
+    """ return current selected language """
+    t = _active.get(currentThread(), None)
+    if t is not None:
+        return t.language()
+    else:
+        from django.conf.settings import LANGUAGE_CODE
+        return LANGUAGE_CODE
+
 def gettext(message):
     """
     This function will be patched into the builtins module to
@@ -171,15 +183,27 @@
     """
     global _accepted
 
+    # check GET, and store selection
+    if request.GET:
+        lang = request.GET.get('django_language', None)
+        if lang is not None:
+            if hasattr(request, 'session'):
+                # store in session
+                request.session['django_language'] = lang
+            else:
+                # store in cookie
+                response.set_cookie('django_language', lang)
+            return lang
+
     if hasattr(request, 'session'):
         lang = getattr(request.session, 'django_language', None)
         if lang is not None:
             return lang
-    
+
     lang = request.COOKIES.get('django_language', None)
     if lang is not None:
         return lang
-    
+
     from django.conf import settings
 
     accept = request.META.get('HTTP_ACCEPT_LANGUAGE', None)
@@ -208,7 +232,7 @@
             if lang == 'en' or os.path.isfile(os.path.join(globalpath, lang, 'LC_MESSAGES', 'django.mo')):
                 _accepted[accept] = lang
                 return lang
-    
+
     return settings.LANGUAGE_CODE
 
 def install():
Index: core/extensions.py
===================================================================
--- core/extensions.py	(revision 767)
+++ core/extensions.py	(working copy)
@@ -34,6 +34,14 @@
         self['user'] = request.user
         self['messages'] = request.user.get_and_delete_messages()
         self['perms'] = PermWrapper(request.user)
+        # add i18n stuff
+        self['LANGUAGE_CODE'] = request.LANGUAGE_CODE
+        try:
+            from django.conf.settings import LANGUAGES
+            self['LANGUAGES'] = LANGUAGES
+        except ImportError:
+            pass
+
         if DEBUG and request.META.get('REMOTE_ADDR') in INTERNAL_IPS:
             self['debug'] = True
             from django.core import db
